import SingletonBase from './base/Singleton';
import { EventType } from './EventEnum';
import { Callback, Callback1, Callback2, Callback3, Callback4 } from './Callback';
import { IBrowserWindow } from './EventInterface';
import { AxiosRequestConfig } from 'axios';
export default class EventManager extends SingletonBase {
    private _listener;
    private _networkListener;
    private _windows;
    mainInit(windows: IBrowserWindow[]): void;
    rendererInit(): void;
    /**
     * 发送网络请求
     * @param options 参数
     */
    sendRequest(options: AxiosRequestConfig<any>): Promise<any>;
    private _addEventListener;
    private _removeEventListener;
    /**
     * 添加事件监听器
     * @param type 事件类型
     * @param cb 回调函数
     */
    addEventListener(type: EventType, cb: Callback): void;
    addEventListener<Arg1>(type: EventType, cb: Callback1<Arg1>): void;
    addEventListener<Arg1, Arg2>(type: EventType, cb: Callback2<Arg1, Arg2>): void;
    addEventListener<Arg1, Arg2, Arg3>(type: EventType, cb: Callback3<Arg1, Arg2, Arg3>): void;
    addEventListener<Arg1, Arg2, Arg3, Arg4>(type: EventType, cb: Callback4<Arg1, Arg2, Arg3, Arg4>): void;
    /**
     * 移除事件监听器
     * @param type 事件类型
     * @param cb 回调函数
     */
    removeEventListener(type: EventType, cb: Callback): void;
    removeEventListener<Arg1>(type: EventType, cb: Callback1<Arg1>): void;
    removeEventListener<Arg1, Arg2>(type: EventType, cb: Callback2<Arg1, Arg2>): void;
    removeEventListener<Arg1, Arg2, Arg3>(type: EventType, cb: Callback3<Arg1, Arg2, Arg3>): void;
    removeEventListener<Arg1, Arg2, Arg3, Arg4>(type: EventType, cb: Callback4<Arg1, Arg2, Arg3, Arg4>): void;
    /**
     * 事件广播
     * @param type 事件类型
     * @param arg1 参数 1
     * @param arg2 参数 2
     * @param arg3 参数 3
     * @param arg4 参数 4
     * @param arg5 参数 5
     */
    broadcast(type: EventType, arg1?: undefined, arg2?: undefined, arg3?: undefined, arg4?: undefined, arg5?: undefined): void;
    broadcast<Arg1>(type: EventType, arg1: Arg1, arg2?: undefined, arg3?: undefined, arg4?: undefined, arg5?: undefined): void;
    broadcast<Arg1, Arg2>(type: EventType, arg1: Arg1, arg2: Arg2, arg3?: undefined, arg4?: undefined, arg5?: undefined): void;
    broadcast<Arg1, Arg2, Arg3>(type: EventType, arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4?: undefined, arg5?: undefined): void;
    broadcast<Arg1, Arg2, Arg3, Arg4>(type: EventType, arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4, arg5?: undefined): void;
    /**
     * 执行渲染进程回调
     * @param type 事件类型
     * @param isMainSend 是否是主线程发送
     * @param args 参数
     */
    private _execute;
    /**
     * 执行主进程回调
     * @param type 事件类型
     * @param args 参数
     */
    private _executeMain;
}
